/***************************************************************************
    begin                : Thu Apr 24 15:54:58 CEST 2003
    copyright            : (C) 2003 by Giuseppe Lipari
    email                : lipari@sssup.it
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifndef __OBSERVER_CONNECTOR_HPP__
#define __OBSERVER_CONNECTOR_HPP__

/**
   An abstract class, it only specifies the interface of a
   ObserverConnector. It is used in Observed class to create lists of
   ObserverConncetors.
*/
class ObserverConnectorInt {
public:
	virtual ~ObserverConnectorInt() {};
	virtual void probe() = 0;
};

/**
   A ObserverConnector is a function object whose only job is to
   "connect" a subject of a certain type (template parameter S) to an
   observer (template parameter O).  The user only needs to call the
   constructor to create a ObserverConnector and add it to a subject.

   The requirements are:
   - the subject must accept pointers to ObserverConnectorInt
   - the subject must call the probe function of each ObserverConnector
   - the observer must implement a method called update(Subject&)
   that will be called by the probe() method

   The class has been provided to allow observers to be passed
   the subject reference. In this way, the update() can be overloaded
   with the subject type, and thus the observer can be notified by different
   subjects at the same time.
*/
template<class Subject, class Observer>
class ObserverConnector : public ObserverConnectorInt {
private:
	Subject &sub_;
	Observer &obs_;
public:
	/**
	 @param s: reference to the subjet
	 @param o: reference to the obeserver
	 */
	ObserverConnector(Subject &s, Observer &o)
		: sub_(s), obs_(o) {}

	virtual ~ObserverConnector() {}

	virtual void probe() {
		obs_.update(sub_);
	}
};
#endif
